Module: Fedipub::ApplicationActor

Extended by:
ActiveSupport::Concern
Included in:
Actor
Defined in:
app/models/concerns/fedipub/application_actor.rb

Overview

Definition for a single local Application Actor, which represents the instance/server itself.

Keys from this actor are used to sign outgoing GET requests.

Constant Summary collapse

APPLICATION_ACTOR_ATTRIBUTES =
{
  username:   '__application',
  actor_type: 'Application',
  entity:     nil,
  local:      true,
}.freeze