Class: Fedipub::Activity
- Inherits:
-
ApplicationRecord
- Object
- ActiveRecord::Base
- ApplicationRecord
- Fedipub::Activity
- Includes:
- HasUuid
- Defined in:
- app/models/fedipub/activity.rb
Overview
Activities can be compared to a log of what happened in the Fediverse.
Activities from local actors ends in the actors outboxes. Activities form distant actors comes from the actor's inbox. We try to only keep activities from local actors, and external activities targetting local actors.
See also:
- https://www.w3.org/TR/activitypub/#outbox
- https://www.w3.org/TR/activitypub/#inbox
Instance Method Summary collapse
-
#undo! ⇒ Fedipub::Activity
Sends an Undo activity to reverse this one.
Methods included from HasUuid
Instance Method Details
#undo! ⇒ Fedipub::Activity
Sends an Undo activity to reverse this one.
Used for unfollowing, unliking, unboosting, etc.
39 40 41 |
# File 'app/models/fedipub/activity.rb', line 39 def undo! Activity.create! actor: actor, action: 'Undo', entity: self, to: to, cc: cc end |